/* * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <phk@FreeBSD.ORG> wrote this file.  As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return.   Poul-Henning Kamp
 * ----------------------------------------------------------------------------
 * $Id: configkit.h,v 1.25 2008/04/29 19:40:09 phk Exp $
 *
 * Parse the configuration and get things going.
 *
 */

#ifndef _CONFIGKIT_H_
#define _CONFIGKIT_H_ 1

#include <sys/queue.h>

struct __ck_node;

struct ck_args {
	struct ck_tree	*cft;
	struct ck_src	*cfs;
	void		*src;
	void		*priv;
	char		**tok;
	int		flag;
	char		*tree;
	char		*line;
	unsigned	output_happened;
};

#define CK_HELP        (1<<0)
#define CK_NO          (1<<1)
#define CK_SHOW        (1<<2)
#define CK_CONF        (1<<3)

#define CK_ARG /*lint -esym(818,cfa) */ struct ck_args *cfa __unused 
#define CK_PASS cfa
#define CK_NOARG NULL

typedef int ck_func_t(CK_ARG);

int ck_printf(CK_ARG, const char *fmt, ...);

/*
 * Config tree construction (ck_tree.c)
 */
struct ck_tree *ck_TreeNew(void);
void ck_TreeAdd(struct ck_tree *cft, struct __ck_node *n);

/* Call further into a tree */
int ck_continue(CK_ARG, struct __ck_node *n, char **argv);

/*
 * User convenience functions.
 */
int ck_WordVar(CK_ARG, const char **arg, char **var);
int ck_UintVar(CK_ARG, unsigned *arg, unsigned *var, unsigned defval);
int ck_RealVar(CK_ARG, double *arg, double *var, double defval, const char *fmt);
int ck_RealVar2(CK_ARG, const char **arg, double *var, char **strvar, double defval, const char *fmt);
int ck_BoolVar(CK_ARG, unsigned *bits, unsigned bit, unsigned defval);
int ck_BoolKeyVar(CK_ARG, const char **key, const char *name, unsigned *bits, unsigned bit, unsigned defval);

/*
 * Instance handling functions
 */

struct ck_instance {
	TAILQ_ENTRY(ck_instance)	next;
	void				*parent;
	void				*obj;
	union {
		unsigned		uint;
		char *			word;
	}				idx;
};

TAILQ_HEAD(ck_instancehead, ck_instance);

#endif /* _CONFIGKIT_H_ */
